"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _webinyValidation = require("webiny-validation");

var _modelError = _interopRequireDefault(require("./modelError"));

var _attributeValue = _interopRequireDefault(require("./attributeValue"));

class Attribute {
  constructor(name, attributesContainer) {
    /**
     * Attribute name.
     */
    this.name = name;
    /**
     * Attribute's parent model instance.
     */

    this.attributesContainer = attributesContainer;
    /**
     * Attribute's current value.
     */

    this.value = new (this.getAttributeValueClass())(this);
    /**
     * If true - updating will be disabled.
     * @var bool
     */

    this.once = false;
    /**
     * If true - setValue will not have any effect.
     * @var bool
     */

    this.dynamic = null;
    /**
     * Marks whether or not this attribute can be stored in a storage.
     * @var bool
     */

    this.toStorage = true;
    /**
     * If true - populate will skip this attribute
     * @var bool
     */

    this.skipOnPopulate = false;
    /**
     * Default value.
     * @var null
     */

    this.defaultValue = null;
    /**
     * Attribute validators.
     * @var string
     */

    this.validators = null;
    /**
     * Custom onSet callback.
     */

    this.onSetCallback = value => value;
    /**
     * Custom onGet callback.
     */


    this.onGetCallback = value => value;
  }
  /**
   * Returns AttributeValue class to be used on construct.
   * @returns {AttributeValue}
   */


  getAttributeValueClass() {
    return _attributeValue.default;
  }
  /**
   * Returns name of attribute
   */


  getName() {
    return this.name;
  }
  /**
   * Returns parent model attributes container
   */


  getParentAttributesContainer() {
    return this.attributesContainer;
  }
  /**
   * Returns model
   */


  getParentModel() {
    return this.getParentAttributesContainer().getParentModel();
  }
  /**
   * Sets data validators, can be a string containing all validators or a callback that throws a ModelError.
   * @param validators
   * @returns {Attribute}
   */


  setValidators(validators = "") {
    this.validators = validators;
    return this;
  }
  /**
   * Returns defined validators or validation callback.
   * @returns {array}
   */


  getValidators() {
    return this.validators;
  }
  /**
   * Returns true if attribute has one or more validators set.
   * @returns {boolean}
   */


  hasValidators() {
    return !!this.validators;
  }

  async getValidationValue() {
    return this.getValue();
  }
  /**
   * Perform validation against currently assigned value.
   * @throws AttributeValidationException
   */


  async validate() {
    const validationValue = this.isSet() ? await this.getValidationValue() : null;
    const validateTypeValue = this.isSet() && !Attribute.isEmptyValue(validationValue);
    validateTypeValue && (await this.validateType(validationValue));
    this.hasValidators() && (await this.validateAttribute(validationValue));
    validateTypeValue && (await this.validateValue(validationValue));
  }
  /**
   * Only used for validating data type only (eg. string must not be send to an attribute that accepts numbers).
   * Will be triggered before data validation by given validators.
   */
  // eslint-disable-next-line


  async validateType(value) {} // Does nothing unless this class is extended and method overridden.
  // Throw an error to signal that validation has failed.

  /**
   * Used to additionally check set data (eg. items in array or to additionally validate set value).
   * Will be triggered after data validation by given validators.
   * @returns {Promise<void>}
   */
  // eslint-disable-next-line


  async validateValue(value) {} // Does nothing unless this class is extended and method overridden.
  // Throw an error to signal that validation has failed.

  /**
   * Used to additionally check set data (eg. items in array or to additionally validate set value).
   * Will be triggered after data validation by given validators.
   * @returns {Promise<void>}
   */


  async validateAttribute(value) {
    let validators = this.getValidators();

    if (typeof validators === "string") {
      try {
        await _webinyValidation.validation.validate(value, validators);
      } catch (e) {
        throw new _modelError.default("Invalid attribute.", _modelError.default.INVALID_ATTRIBUTE, {
          message: e.message,
          value: e.value,
          validator: e.validator
        });
      }
    } else if (typeof validators === "function") {
      await validators(value, this);
    }
  }
  /**
   * Resets attribute - empties value and resets value.set flag, which means setting value will again work in cases setOnce is present.
   */


  reset() {
    this.value.reset();
    return this;
  }
  /**
   * Sets skip on populate - if true, value won't be set into attribute when populate method on parent model instance is called.
   */


  setSkipOnPopulate(flag = true) {
    this.skipOnPopulate = flag;
    return this;
  }
  /**
   * Returns true if this attribute will be skipped on populate.
   */


  getSkipOnPopulate() {
    return this.skipOnPopulate;
  }
  /**
   * Checks if given value is empty or not.
   * @param value
   * @returns {boolean}
   */


  static isEmptyValue(value) {
    return value === null || typeof value === "undefined";
  }
  /**
   * Tells us if the value has been set (flag triggered when setValue is called).
   */


  isSet() {
    return this.value.isSet();
  }
  /**
   * Tells us if value can be set or not (eg. dynamic attributes cannot receive data to be set as an attribute value).
   * @returns {boolean}
   */


  canSetValue() {
    if (this.getDynamic()) {
      return false;
    }

    return !(this.getOnce() && this.isSet());
  }
  /**
   * Sets attribute's value.
   * Some attributes may require async behaviour, that is why we annotate both sync and async return values.
   *
   * @param {any} value A value can be anything, depending on the attribute implementation.
   * @returns {void|Promise<void>}
   */


  setValue(value) {
    if (!this.canSetValue()) {
      return;
    }

    this.value.setCurrent(this.onSetCallback(value));
  }
  /**
   * Returns attribute's value.
   * @returns {*}
   */


  getValue() {
    if (this.dynamic) {
      return this.dynamic(...arguments);
    }

    return this.onGetCallback(this.value.getCurrent(), ...arguments);
  }

  onSet(callback) {
    this.onSetCallback = callback;
    return this;
  }

  onGet(callback) {
    this.onGetCallback = callback;
    return this;
  }

  async getJSONValue() {
    return await this.getValue(...arguments);
  }

  setToStorage(flag = true) {
    this.toStorage = flag;
    return this;
  }

  getToStorage() {
    return this.toStorage;
  }

  async getStorageValue() {
    return this.getValue();
  }

  setStorageValue(value) {
    // We don't want to mark value as dirty.
    this.value.setCurrent(value, {
      skipDifferenceCheck: true,
      forceSetAsClean: true
    });
    return this;
  }
  /**
   * Sets default attribute value.
   */


  setDefaultValue(defaultValue) {
    this.setValue(defaultValue);
    return this;
  }
  /**
   * If set to true, attribute's value won't be overridden if new values are about to be set.
   */


  setOnce(flag = true) {
    this.once = flag;
    return this;
  }
  /**
   * Tells us if attribute value can only be set once.
   */


  getOnce() {
    return this.once;
  }

  setDynamic(callback) {
    this.dynamic = callback;
    return this;
  }

  getDynamic() {
    return this.dynamic;
  }

  expected(expecting, got) {
    throw new _modelError.default(`Validation failed, received ${got}, expecting ${expecting}.`, _modelError.default.INVALID_ATTRIBUTE);
  }

}

var _default = Attribute;
exports.default = _default;
//# sourceMappingURL=attribute.js.map