"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _get2 = _interopRequireDefault(require("lodash/get"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _isObject2 = _interopRequireDefault(require("lodash/isObject"));

var _each2 = _interopRequireDefault(require("lodash/each"));

var _has2 = _interopRequireDefault(require("lodash/has"));

var _webinyDataExtractor = _interopRequireDefault(require("webiny-data-extractor"));

var _defaultAttributesContainer = _interopRequireDefault(require("./defaultAttributesContainer"));

var _modelError = _interopRequireDefault(require("./modelError"));

class Model {
  constructor(params) {
    this.attributes = {};
    this.processing = {
      validation: false,
      dirty: false
    };
    this.attributesContainer = this.createAttributesContainer();

    if (params) {
      if (typeof params === "function") {
        params.call(this);
      } else if (typeof params === "object") {
        // Here, params can be an object, which may contain several options.
        if (typeof params.definition === "function") {
          params.definition.call(this);
        }
      }
    }

    return new Proxy(this, {
      set: (instance, key, value) => {
        const attr = instance.getAttribute(key);

        if (attr) {
          attr.setValue(value);
          return true;
        }

        instance[key] = value;
        return true;
      },
      get: (instance, key) => {
        if (["classId"].includes(key)) {
          return instance.constructor[key];
        }

        const attr = instance.getAttribute(key);

        if (attr) {
          return attr.getValue();
        }

        return instance[key];
      }
    });
  }

  attr(attribute) {
    return this.getAttributesContainer().attr(attribute);
  }

  createAttributesContainer() {
    return new _defaultAttributesContainer.default(this);
  }

  getAttributesContainer() {
    return this.attributesContainer;
  }

  getAttribute(attribute) {
    if ((0, _has2.default)(this.attributes, attribute)) {
      return this.attributes[attribute];
    }
  }

  setAttribute(name, attribute) {
    this.attributes[name] = attribute;
    return this;
  }

  getAttributes() {
    return this.attributes;
  }

  clean() {
    (0, _each2.default)(this.attributes, attribute => {
      attribute.value.isDirty() && attribute.value.clean();
    });
  }

  isDirty() {
    if (this.processing.dirty) {
      return false;
    }

    this.processing.dirty = true;
    let name;

    for (name in this.attributes) {
      const attribute = this.attributes[name];

      if (attribute.value.isDirty()) {
        this.processing.dirty = false;
        return true;
      }
    }

    this.processing.dirty = false;
    return false;
  }

  isClean() {
    return !this.isDirty();
  }
  /**
   * Populates the model with given data.
   */


  populate(data) {
    if (data === null) {
      return this;
    }

    if (!(0, _isObject2.default)(data)) {
      throw new _modelError.default("Populate failed - received data not an object.", _modelError.default.POPULATE_FAILED_NOT_OBJECT);
    }

    (0, _each2.default)(this.attributes, (attribute, name) => {
      if (attribute.getSkipOnPopulate()) {
        return;
      }

      if ((0, _has2.default)(data, name)) {
        attribute.setValue(data[name]);
      }
    });
    return this;
  }
  /**
   * Validates values of all attributes.
   */


  async validate() {
    if (this.processing.validation) {
      return;
    }

    this.processing.validation = true;
    const invalidAttributes = {};
    await Promise.all(Object.keys(this.attributes).map(async name => {
      const attribute = this.attributes[name];

      try {
        await attribute.validate();
      } catch (e) {
        if (e instanceof _modelError.default) {
          invalidAttributes[name] = {
            code: e.code,
            data: e.data,
            message: e.message
          };
        } else {
          invalidAttributes[name] = {
            code: _modelError.default.INVALID_ATTRIBUTE,
            data: null,
            message: e.message
          };
        }
      }
    }));
    this.processing.validation = false;

    if (!(0, _isEmpty2.default)(invalidAttributes)) {
      throw new _modelError.default("Validation failed.", _modelError.default.INVALID_ATTRIBUTES, {
        invalidAttributes
      });
    }
  }

  async toJSON(fields) {
    return await _webinyDataExtractor.default.get(this, fields, {
      onRead: async (data, key) => {
        // Key can accept ":" separated arguments, so we have to make sure those are parsed.
        const received = this.__parseKeyParams(key);

        if (typeof data.getAttribute === "function") {
          if (!data.getAttribute(received.key)) {
            return [received.key];
          }

          return [received.key, await data.getAttribute(received.key).getJSONValue(...received.arguments)];
        }

        return [received.key, await data[received.key]];
      }
    });
  }

  async get(path = "", defaultValue) {
    const steps = typeof path === "string" ? path.split(".") : path;
    let value = this;

    for (let i = 0; i < steps.length; i++) {
      if (!(0, _isObject2.default)(value)) {
        return defaultValue;
      } // Key can accept ":" separated arguments, so we have to make sure those are parsed.


      const received = this.__parseKeyParams(steps[i]);

      if (typeof value.getAttribute === "function") {
        if (value.getAttribute(received.key)) {
          value = await value.getAttribute(received.key).getValue(...received.arguments);
        } else {
          return defaultValue;
        }
      } else {
        value = await value[steps[i]];
      }
    }

    return value;
  }

  __parseKeyParams(key) {
    const received = {
      arguments: key.split(":"),
      key: ""
    };
    received.key = received.arguments.shift();
    return received;
  }

  async set(path, value) {
    const steps = path.split(".");
    const lastStep = steps.pop();
    const model = await this.get(steps);

    if (model && model.getAttribute) {
      const attribute = model.getAttribute(lastStep);

      if (attribute) {
        return attribute.setValue(value);
      }
    }
  }
  /**
   * Returns data that is suitable for latter saving in a storage layer (database, caching etc.). This is useful because attributes can
   * have different values here (eg. only ID sometimes is needed) and also some attributes don't even need to be saved in the storage,
   * which is most often the case with dynamic attributes.
   */


  async toStorage() {
    const json = {};

    for (let name in this.getAttributes()) {
      const attribute = this.getAttribute(name); // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.

      if (attribute.getToStorage() && attribute.value.isDirty() && !attribute.getDynamic()) {
        // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
        json[name] = await attribute.getStorageValue();
      }
    }

    return json;
  }

  populateFromStorage(data) {
    if (data === null) {
      return this;
    }

    if (!(0, _isObject2.default)(data)) {
      throw new _modelError.default("Populate failed - received data not an object.", _modelError.default.POPULATE_FAILED_NOT_OBJECT);
    }

    let name;

    for (name in this.getAttributes()) {
      const attribute = this.getAttribute(name);
      (0, _has2.default)(data, name) && // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
      attribute.getToStorage() && // $FlowFixMe - we can be sure we have attribute because it's pulled from list of attributes, using getAttributes() method.
      attribute.setStorageValue(data[name]);
    }

    return this;
  }

  static isModelInstance(value) {
    return !!(0, _get2.default)(value, "constructor.classId");
  }

  static isModelClass(value) {
    return !!(0, _get2.default)(value, "classId");
  }

  static isInstanceOf(instance, instanceClass) {
    return (0, _get2.default)(instance, "constructor.classId") === (0, _get2.default)(instanceClass, "classId");
  }

}

Model.classId = "";
var _default = Model;
exports.default = _default;
//# sourceMappingURL=model.js.map